import path from "path";
import { isStaticMetadataRoute } from "../../../lib/is-app-route-route";
function getContentType(resourcePath) {
    const filename = path.basename(resourcePath);
    const [name, ext] = filename.split(".");
    if (name === "favicon" && ext === "ico") return "image/x-icon";
    if (name === "sitemap") return "application/xml";
    if (name === "robots") return "text/plain";
    return "text/plain";
}
// `import.meta.url` is the resource name of the current module.
// When it's static route, it could be favicon.ico, sitemap.xml, robots.txt etc.
const nextMetadataRouterLoader = function() {
    const { resourcePath  } = this;
    const code = isStaticMetadataRoute(resourcePath) ? `import { NextResponse } from 'next/server'
import fs from 'fs'
import path from 'path'
import { fileURLToPath } from 'url'

const buffer = fs.readFileSync(fileURLToPath(import.meta.url))
const contentType = ${JSON.stringify(getContentType(resourcePath))}

export function GET() {
  return new NextResponse(buffer, {
    status: 200,
    headers: {
      'Content-Type': contentType,
    },
  })
}

export const dynamic = 'force-static'
` : `export { default as GET } from ${JSON.stringify(resourcePath)}`;
    return code;
};
export default nextMetadataRouterLoader;

//# sourceMappingURL=next-metadata-route-loader.js.map